import React from 'react'
import { get } from 'lodash'
import { connect } from 'react-redux'
import PropTypes from 'prop-types'
import { Button, Icon, th } from '@pubsweet/ui'
import styled, { css, withTheme } from 'styled-components'
import { compose, getContext, withHandlers } from 'recompose'
import {
  withModal,
  ConfirmationModal,
} from 'pubsweet-component-modal/src/components'
import AuthorsWithTooltip from 'pubsweet-component-manuscript/src/molecules/AuthorsWithTooltip'
// import { AuthorsWithTooltip } from '@pubsweet/ui'

import ZipFiles from '../Files/ZipFiles'
import { InviteReviewers } from '../Reviewers/'
import { currentUserIs } from '../../redux/users'
import { selectInvitation } from '../../redux/reviewers'
import { ReviewerDecision, HandlingEditorSection } from './'
import { parseVersion, parseJournalIssue, mapStatusToLabel } from './../utils'
import { canMakeRecommendation } from '../../../../component-faraday-selectors/src'

import { ReviewerBreakdown } from '../Invitations'
import { Recommendation } from '../MakeRecommendation'

const DashboardCard = ({
  isHE,
  theme,
  journal,
  history,
  project,
  version,
  invitation,
  currentUser,
  deleteProject,
  showAbstractModal,
  canInviteReviewers,
  showConfirmationModal,
  canMakeRecommendation,
  ...rest
}) => {
  const { submitted, title, type } = parseVersion(version)
  const metadata = get(version, 'metadata')
  const files = get(version, 'files')
  const customId = project.customId || project.id.split('-')[0]
  const hasFiles = files ? Object.values(files).some(f => f.length > 0) : false
  const journalIssueType = parseJournalIssue(journal, metadata)
  const manuscriptMeta = `${type} - ${
    journalIssueType ? journalIssueType.label : 'N/A'
  }`

  return version ? (
    <Card id={customId}>
      <ListView>
        <Top>
          <LeftDetails flex={6}>
            <ManuscriptId>{`ID ${customId}`}</ManuscriptId>
            <Title
              title={title}
              dangerouslySetInnerHTML={{ __html: title }} // eslint-disable-line
            />
          </LeftDetails>
          <RightDetails flex={2}>
            {canMakeRecommendation && (
              <Recommendation
                collectionId={project.id}
                fragmentId={version.id}
                modalKey={`recommend-${version.id}`}
              />
            )}
            <ZipFiles
              archiveName={`ID-${project.customId}`}
              collectionId={project.id}
              disabled={!hasFiles}
              fragmentId={version.id}
            >
              <ClickableIcon disabled={!hasFiles}>
                <Icon>download</Icon>
              </ClickableIcon>
            </ZipFiles>
            {!project.status && (
              <ActionButtons
                data-test="button-resume-submission"
                onClick={() =>
                  history.push(
                    `/projects/${project.id}/versions/${version.id}/submit`,
                  )
                }
              >
                RESUME SUBMISSION
              </ActionButtons>
            )}
          </RightDetails>
        </Top>
        <Bottom>
          <LeftDetails flex={3}>
            <Status>{mapStatusToLabel(project)}</Status>
            <DateField>{submitted || ''}</DateField>
          </LeftDetails>
          <RightDetails flex={4}>
            <ManuscriptType title={manuscriptMeta}>
              {manuscriptMeta}
            </ManuscriptType>
            {project.status ? (
              <Details
                data-test="button-details"
                onClick={() =>
                  history.push(
                    `/projects/${project.id}/versions/${version.id}/details`,
                  )
                }
              >
                Details
                <Icon color={theme.colorPrimary}>chevron-right</Icon>
              </Details>
            ) : (
              <Details
                data-test="button-cancel-submission"
                onClick={showConfirmationModal}
              >
                Delete
              </Details>
            )}
          </RightDetails>
        </Bottom>
      </ListView>
      {project.status && (
        <DetailsView>
          <Top>
            <AuthorsWithTooltip authors={project.authors} />
          </Top>
          <Bottom>
            <LeftDetails flex={4}>
              <HandlingEditorSection
                currentUser={currentUser}
                project={project}
              />
            </LeftDetails>
            {canInviteReviewers() && (
              <RightDetails flex={4}>
                <ReviewerBreakdown
                  collectionId={project.id}
                  compact
                  versionId={version.id}
                />
                <InviteReviewers
                  modalKey={`invite-reviewers-${project.id}`}
                  project={project}
                  version={version}
                />
              </RightDetails>
            )}
            {invitation && (
              <RightDetails flex={4}>
                <ReviewerText>Invited to review</ReviewerText>
                <ReviewerDecision
                  invitation={invitation}
                  modalKey={`reviewer-decision-${project.id}`}
                  project={project}
                />
              </RightDetails>
            )}
          </Bottom>
        </DetailsView>
      )}
    </Card>
  ) : null
}

// const isHEToManuscript = (state, collectionId) => {
//   const currentUserId = get(state, 'currentUser.user.id')
//   const collections = get(state, 'collections') || []
//   const collection = collections.find(c => c.id === collectionId) || {}
//   const collectionInvitations = get(collection, 'invitations') || []
//   const userInvitation = collectionInvitations.find(
//     i => i.role === 'handlingEditor' && i.userId === currentUserId,
//   )

//   return userInvitation ? userInvitation.isAccepted : false
// }

export default compose(
  getContext({ journal: PropTypes.object, currentUser: PropTypes.object }),
  withTheme,
  withModal({
    modalKey: 'cancelManuscript',
    modalComponent: ConfirmationModal,
  }),
  connect((state, { project }) => ({
    isHE: currentUserIs(state, 'handlingEditor'),
    invitation: selectInvitation(state, project.id),
    canMakeRecommendation: canMakeRecommendation(state, project),
  })),

  withHandlers({
    canInviteReviewers: ({ currentUser, project }) => () => {
      const handlingEditor = get(project, 'handlingEditor')
      const isAdmin = get(currentUser, 'admin')
      const isEic = get(currentUser, 'editorInChief')
      const isAccepted = get(handlingEditor, 'isAccepted')
      const heId = get(handlingEditor, 'id')
      return isAccepted && (currentUser.id === heId || isAdmin || isEic)
    },
    showConfirmationModal: ({
      deleteProject,
      showModal,
      hideModal,
      setModalError,
      project,
    }) => () => {
      showModal({
        title: 'Are you sure you want to delete this submission?',
        confirmText: 'Delete',
        onConfirm: () => {
          deleteProject(project).then(hideModal, e => {
            setModalError(
              get(JSON.parse(e.response), 'error') ||
                'Oops! Something went wrong!',
            )
          })
        },
      })
    },
  }),
)(DashboardCard)

// #region styled-components
const defaultText = css`
  color: ${th('colorText')};
  font-family: ${th('fontReading')};
  font-size: ${th('fontSizeBaseSmall')};
`

const ReviewerText = styled.div`
  ${defaultText};
  margin-right: ${th('subGridUnit')};
  text-transform: uppercase;
`

const ActionButtons = styled(Button)`
  ${defaultText};
  align-items: center;
  background-color: ${th('colorPrimary')};
  color: ${th('colorTextReverse')};
  display: flex;
  padding: 4px 8px;
  text-align: center;
  height: calc(${th('subGridUnit')}*5);
`

const LeftDetails = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: flex-start;
  align-items: center;
  flex: ${({ flex }) => flex || 1};
  flex-wrap: wrap;
`

const RightDetails = styled.div`
  display: flex;
  flex-direction: row;
  justify-content: flex-end;
  align-items: center;
  flex: ${({ flex }) => flex || 1};
  flex-wrap: wrap;
`

const DetailsView = styled.div`
  align-items: center;
  border-top: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  width: 100%;
`

const ListView = styled.div`
  align-items: center;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  width: 100%;
`

const ManuscriptId = styled.div`
  ${defaultText};
  margin-right: 8px;
  text-align: left;
  text-transform: uppercase;
  white-space: nowrap;
  flex: 1;
`

const Details = styled.div`
  align-items: center;
  cursor: pointer;
  display: flex;
  ${defaultText};
  margin-left: 16px;
  text-decoration: underline;
  text-align: center;
  text-transform: uppercase;
  white-space: nowrap;
`

const ClickableIcon = styled.div`
  cursor: ${({ disabled }) => (disabled ? 'not-allowed' : 'pointer')};
  margin: 0 7px;

  svg {
    stroke: ${({ disabled, theme }) =>
      disabled ? theme.colorBackgroundHue : theme.colorPrimary};
  }
`

const Card = styled.div`
  align-items: center;
  border: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  justify-content: flex-start;
  margin-bottom: 10px;
  background-color: ${th('backgroundColorReverse')};
`

const Top = styled.div`
  display: flex;
  flex-direction: row;
  align-items: center;
  padding: 10px;
  width: 100%;
  box-sizing: border-box;
  flex-wrap: wrap;
`
const Bottom = styled.div`
  display: flex;
  flex-direction: row;
  padding: 0 10px 10px 10px;
  width: 100%;
  box-sizing: border-box;
  justify-content: space-between;
  flex-wrap: wrap;
`

const ManuscriptType = styled.div`
  ${defaultText};
  padding: 6px 4px;
  margin-left: 10px;
  text-align: right;
  text-transform: capitalize;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
  flex: 1;
`

const Title = styled.div`
  ${defaultText};
  font-family: ${th('fontHeading')};
  color: ${th('colorPrimary')};
  font-weight: bold;
  text-align: left;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
  flex: 10;
`

const Status = styled.div`
  border: ${th('borderDefault')};
  ${defaultText};
  font-weight: bold;
  padding: 0 0.5em;
  text-align: left;
  text-transform: capitalize;
  line-height: 1.5;
  color: ${th('colorPrimary')};
`

const DateField = styled.span`
  ${defaultText};
  margin: 0 ${th('subGridUnit')};
  text-align: left;
`

// #endregion
