import React from 'react'
import moment from 'moment'
import { pick, get } from 'lodash'
import { connect } from 'react-redux'
import { th, Icon } from '@pubsweet/ui'
import styled, { withTheme } from 'styled-components'
import { compose, withHandlers, withProps } from 'recompose'
import {
  ConfirmationModal,
  withModal2,
} from 'pubsweet-component-modal/src/components'

import {
  revokeReviewer,
  inviteReviewer,
  selectFetchingInvite,
  selectReviewersError,
  getCollectionReviewers,
} from '../../redux/reviewers'

const ResendRevoke = withTheme(
  ({ theme, showConfirmResend, showConfirmRevoke, status }) => (
    <ActionButtons>
      <div onClick={showConfirmResend}>
        <Icon color={theme.colorPrimary}>refresh-cw</Icon>
      </div>
      {status === 'pending' && (
        <div onClick={showConfirmRevoke}>
          <Icon color={theme.colorPrimary}>x-circle</Icon>
        </div>
      )}
    </ActionButtons>
  ),
)

const TR = ({
  reviewer: r,
  renderAcceptedLabel,
  index,
  renderTimestamp,
  showConfirmResend,
  showConfirmRevoke,
}) => {
  const submittedOn = get(r, 'review.submittedOn')
  return (
    <Row>
      <td>
        <ReviewerName>{r.name}</ReviewerName>{' '}
        {r.status === 'accepted' && (
          <AcceptedReviewer>{renderAcceptedLabel(index)}</AcceptedReviewer>
        )}
      </td>
      <td>{r.invitedOn ? renderTimestamp(r.invitedOn) : ''}</td>
      <td>
        <StatusText>{r.status === 'accepted' ? 'Agreed' : r.status}</StatusText>
        <DateText>
          {r.respondedOn ? `: ${renderTimestamp(r.respondedOn)}` : ''}
        </DateText>
      </td>
      <td>{submittedOn ? `${renderTimestamp(submittedOn)}` : ''}</td>
      <td width={100}>
        {r.status === 'pending' && (
          <ResendRevoke
            showConfirmResend={showConfirmResend(r)}
            showConfirmRevoke={showConfirmRevoke(r.invitationId)}
            status={r.status}
          />
        )}
      </td>
    </Row>
  )
}
const ReviewersDetailsList = ({
  renderAcceptedLabel,
  reviewers,
  showConfirmResend,
  showConfirmRevoke,
  renderTimestamp,
}) =>
  reviewers.length > 0 ? (
    <Root>
      <ScrollContainer>
        <Table>
          <thead>
            <tr>
              <td> Full Name </td>
              <td> Invited On</td>
              <td> Responded On</td>
              <td> Submitted On</td>
              <td />
            </tr>
          </thead>
          <tbody>
            {reviewers.map((r, index) => (
              <TR
                index={index}
                key={r.email}
                renderAcceptedLabel={renderAcceptedLabel}
                renderTimestamp={renderTimestamp}
                reviewer={r}
                showConfirmResend={showConfirmResend}
                showConfirmRevoke={showConfirmRevoke}
              />
            ))}
          </tbody>
        </Table>
      </ScrollContainer>
    </Root>
  ) : (
    <div> No reviewers details </div>
  )

const ModalWrapper = compose(
  connect(state => ({
    fetchingInvite: selectFetchingInvite(state),
    modalError: selectReviewersError(state),
  })),
)(({ fetchingInvite, ...rest }) => (
  <ConfirmationModal {...rest} isFetching={fetchingInvite} />
))

export default compose(
  connect(null, { inviteReviewer, revokeReviewer, getCollectionReviewers }),
  withModal2(props => ({
    modalComponent: ModalWrapper,
  })),
  withProps(({ reviewers = [] }) => ({
    firstAccepted: reviewers.findIndex(r => r.status === 'accepted'),
  })),
  withHandlers({
    renderTimestamp: () => timestamp => {
      const today = moment()
      const stamp = moment(timestamp)
      const duration = moment.duration(today.diff(stamp))

      if (duration.asDays() < 1) {
        return `${duration.humanize()} ago`
      }
      return stamp.format('DD.MM.YYYY')
    },
    renderAcceptedLabel: ({ firstAccepted }) => index =>
      `Reviewer ${index - firstAccepted + 1}`,
  }),
  withHandlers({
    showConfirmResend: ({
      showModal,
      hideModal,
      setModalError,
      goBackToReviewers,
      inviteReviewer,
      collectionId,
      getCollectionReviewers,
    }) => reviewer => () => {
      showModal({
        title: 'Resend reviewer invite',
        confirmText: 'Resend',
        onConfirm: () => {
          inviteReviewer(
            pick(reviewer, ['email', 'firstName', 'lastName', 'affiliation']),
            collectionId,
          )
            .then(() => getCollectionReviewers(collectionId))
            .then(hideModal)
        },
        onCancel: hideModal,
      })
    },
    showConfirmRevoke: ({
      showModal,
      hideModal,
      setModalError,
      goBackToReviewers,
      revokeReviewer,
      collectionId,
      getCollectionReviewers,
    }) => invitationId => () => {
      showModal({
        title: 'Unassign Reviewer',
        confirmText: 'Unassign',
        onConfirm: () => {
          revokeReviewer(invitationId, collectionId)
            .then(() => getCollectionReviewers(collectionId))
            .then(hideModal)
        },
        onCancel: hideModal,
      })
    },
  }),
)(ReviewersDetailsList)

// #region styled-components
const ReviewerEmail = styled.span`
  font-size: ${th('fontSizeBaseSmall')};
  color: ${th('colorPrimary')};
  font-family: ${th('fontReading')};
`

const ReviewerName = ReviewerEmail.extend`
  text-decoration: underline;
`

const AcceptedReviewer = ReviewerEmail.extend`
  font-weight: bold;
  margin-left: ${th('subGridUnit')};
`

const StatusText = ReviewerEmail.extend`
  text-transform: uppercase;
`

const DateText = ReviewerEmail.extend``

const ScrollContainer = styled.div`
  align-self: stretch;
  flex: 1;
  overflow: auto;
`
const Root = styled.div`
  align-items: stretch;
  align-self: stretch;
  background-color: ${th('backgroundColorReverse')};
  display: flex;
  flex-direction: column;
  justify-content: flex-start;
  height: 25vh;
`

const Table = styled.table`
  border-spacing: 0;
  border-collapse: collapse;
  width: 100%;

  & thead tr {
    border-bottom: ${th('borderDefault')};
    color: ${th('colorPrimary')};
    font-family: ${th('fontReading')};
    font-size: ${th('fontSizeBaseSmall')};
    font-weight: bold;
    height: 40px;
    text-align: left;
    line-height: 1.5;
  }
`
const Row = styled.tr`
  border-bottom: ${th('borderDefault')};
  color: ${th('colorPrimary')};
  font-family: ${th('fontReading')};
  font-size: ${th('fontSizeBaseSmall')};
  height: 40px;
  text-align: left;
  line-height: 1.5;

  &:hover {
    background-color: ${th('backgroundColor')};
  }
`

const ActionButtons = styled.div`
  align-items: center;
  display: flex;
  flex: 1;
  justify-content: space-evenly;
  opacity: 0;

  div {
    cursor: pointer;
  }

  ${Row}:hover & {
    opacity: 1;
  }
`
// #endregion
